"""Object representing the various types of names.  Formatted as dictionaries
mapping plaintext versions of names to the number of occurrences of that names
"""

#!/usr/bin/python3

# Mike Canoy
# Authored 5/9/2016

personal = {}

geographic = {}

field = {}
